// ===================================================================
// $Id: brdfwrap.cpp,v 1.3 2005/11/02 12:59:01 xkrivanj Exp $
//
// brdfwrap.cpp
//      
// Initial coding by Jaroslav Krivanek, December 2003
//

// GOLEM headers
#include "config.h"
#include "brdfwrap.h"
#include "scene.h"
#include "surface.h"
#include "world.h"

// standard headers

__BEGIN_GOLEM_SOURCE

CBRDFSurfaceWrapper::CBRDFSurfaceWrapper(const CSurface* surf)
: _surf  (surf),
  _tmpCol(world->_scene->stdValues.GetDefaultWeightType())
{
	assert(surf);
	_hitInfo.SetPoint (CVector3D(0,0,0));
	_hitInfo.SetNormal(CVector3D(0,0,1));
	_hitInfo.unormal=CVector3D(CVector3D(0,0,1));
	_hitInfo.normalValid=true;
	_hitInfo.SetObject(0);
    _hitInfo.SetFlags(CHitPointInfo::EE_IdealMirrorValid);
	_channels = _tmpCol.GetCount();
	_hitInfo.ray = &_ray;
	_ray.Init(CVector3D(0,0,0), CVector3D(0,0,0), DUMMY_RAY, 0, 0, 0, true);
}

CBRDFSurfaceWrapper::~CBRDFSurfaceWrapper() 
{
}

inline CVector3D _reflectedDir(const CVector3D& dir, const CVector3D& N)
{
    return N * 2.0 * DotProd(N, -dir) + dir;
}

// reflected dir for normal = CVector3D(0,0,1)
inline CVector3D _reflectedDir(const CVector3D& dir)
{
	return CVector3D(dir.x,dir.y,-dir.z);
}

float CBRDFSurfaceWrapper::Eval(const float *finDir, const float* foutDir) const
{
  CVector3D inDir(finDir[0],finDir[1],finDir[2]);
  CVector3D outDir(foutDir[0],foutDir[1],foutDir[2]);

	_hitInfo.SetDotProd( -inDir.z );
	_hitInfo.idealMirrorDir = _reflectedDir(-inDir);
	_ray.dir = -inDir;
	_ray.loc =  inDir;
	if( _surf->EvaluateBSDF(_tmpCol,_hitInfo,outDir,EE_ALL) )
    return _tmpCol.Luminance() * inDir.z;
  else
    return 0;
}

void CBRDFSurfaceWrapper::EvalCol(float* col, const float* finDir, const float* foutDir) const
{
  CVector3D inDir(finDir[0],finDir[1],finDir[2]);
  CVector3D outDir(foutDir[0],foutDir[1],foutDir[2]);

	_hitInfo.SetDotProd( -outDir.z );
	_hitInfo.idealMirrorDir = _reflectedDir(-outDir);
	_ray.dir = -outDir;
	_ray.loc =  outDir;
	if(_surf->EvaluateBSDF(_tmpCol,_hitInfo,inDir,/*EE_ALL*/ EE_REFLECTED_GLOSSY))
    for(int i=0;i<_channels;i++)
      col[i] = _tmpCol.Get(i) * inDir.z;
  else 
    for(int i=0;i<_channels;i++)
      col[i] = 0;
}

__END_GOLEM_SOURCE
