// ===================================================================
// $Id: brdfwrap.h,v 1.4 2005/11/02 12:58:42 xkrivanj Exp $
//
// brdfwrap.h
//    This is a CSurface wrapper making it usable for a 
//    C(H)SHBRDF interface.
//
// Class: 
//
// Initial coding by Jaroslav Krivanek, December 2003.

#ifndef __BRDFWRAP_H__
#define __BRDFWRAP_H__

// GOLEM headers
#include "configh.h"
#include "color.h"
#include "hminfo.h"

// SPHAR headers
#include "brdf.h"

__BEGIN_GOLEM_HEADER

// forward declarations
class CSurface;
class CRay;

/// Wrapper class for CSurface class
class CBRDFSurfaceWrapper :
  public CBRDF<float,float>
{
  /// surface interface that we are wrapping
  const CSurface *_surf;

  /// auxiliary ray for BRDF evaluation
  mutable CRay _ray;

  /// an auxiliary hitInfo for BRDF evaluation (instead of a local variable)
  mutable CHitPointInfo _hitInfo;

  /// an auxiliary color value for BRDF evaluation (instead of a local variable)
  mutable CValues       _tmpCol;

  // number of color channels
  int _channels;

public:
  // constructor and destructor
  CBRDFSurfaceWrapper(const CSurface* surf);
  virtual ~CBRDFSurfaceWrapper();

  // member functions
  virtual float Eval   (const float* dirIn, const float* dirOut) const;
  virtual void  EvalCol(float* col, const float *dirIn, const float* dirOut)
    const;
};


__END_GOLEM_HEADER

#endif // __BRDFWRAP_H__
