// ===================================================================
// $Id: cachmode.cpp,v 1.1 2005/11/02 12:59:02 xkrivanj Exp $
//
// cachmode.cpp
//      
// Initial coding by Jaroslav Krivanek, March 2005

// GOLEM headers
#include "config.h"
#include "cachmode.h"
#include "errorg.h"
#include "environ.h"

__BEGIN_GOLEM_SOURCE

// --------------------------------------------------------------------------
//   CCachingMode::SetFromString()
// --------------------------------------------------------------------------
CCachingMode::EMode 
CCachingMode::SetFromString(const std::string& s)
{
  if (s=="normal")
    return EE_Normal;
  else if(s=="always_sample")
    return EE_AlwaysSample;
  else if(s=="always_insert")
    return EE_AlwaysInsert;
  else if(s=="always_read")
    return EE_AlwaysRead;
  else if(s=="same")
    return EE_Same;
  else
    return EE_Error;
}

// --------------------------------------------------------------------------
//   CCachingMode::ReportError()
// --------------------------------------------------------------------------
void 
CCachingMode::ReportError(const std::string& name,
                          const std::string& s,
                          bool sameAllowed)
{
    FATAL << "Invalid " << name << " '" << s << "'.\n";
    FATAL << "Possible values are:\n";
    FATAL << "    'always_sample'\n";
    FATAL << "    'always_insert'\n";
    FATAL << "    'always_read'\n";
    FATAL << "    'normal'";
    if(sameAllowed) {
      FATALW << "\n";
      FATAL << "    'same'";
    }
    FATALW << std::endl;
}

// --------------------------------------------------------------------------
//   CCachingMode::ReadCachingMode()
// --------------------------------------------------------------------------
bool 
CCachingMode::ReadCachingMode(const CEnvironment& env,
                              const std::string& prefix,
                              EMode &presampleModePrim,
                              EMode &presampleModeHi,
                              EMode &renderModePrim,
                              EMode &renderModeHi)
{
  std::string str;

  // (Ir)radianceCache.presampleModePrim
  env.GetString(prefix+".presampleModePrim", str);
  presampleModePrim = CCachingMode::SetFromString(str);
  if(presampleModePrim==CCachingMode::EE_Error||
    presampleModePrim==CCachingMode::EE_Same) 
  {
    CCachingMode::ReportError("IrradianceCache.presampleModePrim", str, false);
    return true; // error
  }
  // (Ir)radianceCache.presampleModeHi
  env.GetString(prefix+".presampleModeHi", str);
  presampleModeHi = CCachingMode::SetFromString(str);
  if(presampleModeHi==CCachingMode::EE_Error) {
    CCachingMode::ReportError("IrradianceCache.presampleModeHi", str, true);
    return true; // error
  }
  if(presampleModeHi==CCachingMode::EE_Same)
    presampleModeHi = presampleModePrim;
    
  // (Ir)radianceCache.renderModePrim
  env.GetString(prefix+".renderModePrim", str);
  renderModePrim = CCachingMode::SetFromString(str);
  if(renderModePrim==CCachingMode::EE_Error||
    renderModePrim==CCachingMode::EE_Same) 
  {
    CCachingMode::ReportError("IrradianceCache.renderModePrim", str, false);
    return true; // error
  }
  // (Ir)radianceCache.renderModeHi
  env.GetString(prefix+".renderModeHi", str);
  renderModeHi = CCachingMode::SetFromString(str);
  if(renderModeHi==CCachingMode::EE_Error) {
    CCachingMode::ReportError("IrradianceCache.renderModeHi", str, true);
    return true; // error
  }
  if(renderModeHi==CCachingMode::EE_Same)
    renderModeHi = renderModePrim;

  return false; // ok
}



__END_GOLEM_SOURCE
