// ===================================================================
// $Id: icsplati.h,v 1.1 2005/11/02 12:58:48 xkrivanj Exp $
//
// icsplati.h
//   Irradiance cache splatter for irregular point sets (based on kd-tree).
//   
// REPLACEMENT_STRING
//
// Initial coding by Jaroslav Krivanek (August, 2005).

#ifndef __ICSPLATI_H__
#define __ICSPLATI_H__

// GOLEM headers
#include "configh.h"
#include "icsplat.h"
#include "kdtmpl.h"
// #include "kdtmplex.h"

#include "timer.h"

// standard headers

__BEGIN_GOLEM_HEADER

// forward declarations

#define SPLATTER_KDTREE CKdTreeTmplPtr
//#define SPLATTER_KDTREE CKdTreeTmplPtrEx

/// Splatter for irregular shading points (not only pixel centers).
class CICSplatterIrregular : public CICSplatter {

public:

  typedef SPLATTER_KDTREE<const CHitInfoLite>::CPointAndDist CNearestHitInfoLite;
  typedef growvector<CNearestHitInfoLite> CNearestPointArray;
  typedef SPLATTER_KDTREE<const CHitInfoLite>::CNearestPoints<CNearestPointArray>
    CNearestPointStruct;

  /// Construct splatter and searching structures.
  /**
   @param  numPoints (in)
      Number of shading points.
   @param  hitPoints (in)
      Array of shading points. All shading points whose flag pass the 
      'handleFlag' must point to additional
      data structure, which serves as the canvas pixel for splatting.
      This structure depends on the splatting operation which will be performed.
   @param  handleFlag (in)
      Only those points from the array will be considered that pass this flag.
  */
  CICSplatterIrregular(
    const CHitInfoLite *hitPoints,
    unsigned numHitPoints,
    unsigned numPixels,
    unsigned handleFlag);

  ~CICSplatterIrregular();

  /// Find all hit points in sphere around P and put them in 'nearestPoints'
  void FindInSphereLinear(CNearestPointStruct &nearestPoints);

  /// Find all hit points in sphere around P and put them in 'nearestPoints'
  inline void FindInSphere(CNearestPointStruct &nearestPoints)
  {
    treeTimer.Start();
    // FindInSphereLinear(nearestPoints);
    _kdTree.LocatePoints(nearestPoints);
    treeTimer.Stop();
  }

  inline const CNearestPointArray& FindInSphere(const CVector3D &P,float r)
  {
    // _nearestPoints.clear();
    _nearestPoints.Init(P,r);
    FindInSphere(_nearestPoints);
    return _nearestPoints.list;
  }

  /// Splat irradiance record (irradiance + weight). Return the number of contribs.
  template<bool jitter>
  unsigned Splat(const CIrradianceRecord *rec);

  template<bool jitter>
  unsigned Splat(const CIrradianceList &recList);

  /// Fill pixel list in a hiararchical order.
  template<class TPixList>
  void FillPointList(TPixList& list);

  /// Return the hit points array.
  const CHitInfoLite* GetHitPoints() const {return GetHitInfoImage();}

public: // data

  CTimer treeTimer;

protected:

  /// array for query answers
  // CNearestPointArray _nearestPoints;
  CNearestPointStruct  _nearestPoints;

  SPLATTER_KDTREE<const CHitInfoLite> _kdTree;
};

// --------------------------------------------------------------------------
//  CICSplatterIrregular::FillPixelList()
// --------------------------------------------------------------------------
template<class TPixList>
void
CICSplatterIrregular::FillPointList(TPixList& list)
{
  _kdTree.TraverseBF(list);
 /*
  unsigned numPoints = GetNumHitPoints();

  int imgWidth  = ceil(sqrt((float)numPoints));
  int imgHeight = imgWidth;
  while(imgWidth*imgHeight<(int)numPoints)
    imgWidth++;

  // Using pascal's image traversal algorithm.
  int step = pow2ge(Max(imgWidth,imgHeight));
  int effectivestepx = step; 
  int startx=0; 
  int rownumber=0,  evenstart=0; 
  int level = 0;
  int x,y;

  while(step>=1) {
    for(y=0, rownumber=0; y < imgHeight; y += step, rownumber++) {	
      if (rownumber%2==0) startx = evenstart;
      else startx = 0;        
      effectivestepx = step+startx;
      for(x=startx; x < imgWidth; x += effectivestepx )  {
        unsigned idx = y*imgWidth+x;
        if(idx<numPoints) {
          //////////////////////////////////////////////
          if(IsHandled(GetHitPoints()[idx])) // yes to radiance caching
            list.push(idx);
          //////////////////////////////////////////////
        }
      }
    }
    step /= 2;
    evenstart = step;
  }
 */
}

#undef SPLATTER_KDTREE 

__END_GOLEM_HEADER

#endif // __ICSPLATI_H__
