// ===================================================================
// $Id: rcrecord.h,v 1.1 2005/11/02 12:58:48 xkrivanj Exp $
//
// rcrecord.h
//   Radiance cache record definition
//   
// REPLACEMENT_STRING
//
// Initial coding by Jaroslav Krivanek (August, 2005).

#ifndef __RCRECORD_H__
#define __RCRECORD_H__

// GOLEM headers
#include "configh.h"
#include "icrecord.h"

// SPHAR headers
#include "shvector.h"

__BEGIN_GOLEM_HEADER

// forward declarations
class CRadianceRecord;
template<class T> class CRCOctreeNode;

/// Typedef for list of irradiance records.
typedef CRecordList<CRadianceRecord> CRadianceList;

// --------------------------------------------------------------------------
//  class CIrradianceRecord 
// --------------------------------------------------------------------------
class CRadianceRecord : public CICRCRecord<CRadianceRecord> {

public:

  /// Constructor of the cache entry.
   CRadianceRecord(
    const   CVector3D &P,
		const   CVector3D &U,
		const   CVector3D &V,
		const   CVector3D &N,
		int     numChannels,
		int     order,
		bool    useGradients,
    float  a,
		float  R,
    float  unclampedR,
    float  Rminus,
    float  Rplus,
    float  rotMult,
    float  stdDev,
    void    *data) : 
   CICRCRecord<CRadianceRecord>(P,N,a,R,unclampedR,Rminus,Rplus,rotMult,stdDev,
     data),
     _coeffs(numChannels, order),
     _xderivative(numChannels, useGradients ? order : 0),
     _yderivative(numChannels, useGradients ? order : 0),
     _U(U),
     _V(V)
   { 
   }
   
   ~CRadianceRecord() {}

   /*
   const CValues&   GetIrradiance()       const { return _E; }
   const CValues&   GetE()                const { return _E; }
   const float&    GetStdDev()           const { return _stdDev; }
   bool   GradientsUsed()           const { return _rotGrad!=0; }
   const CVector3D* GetRotGrad()           const { return _rotGrad;   }
   const CVector3D* GetTransGrad()         const { return _transGrad; }
   const CVector3D& GetRotGrad(int i)      const { return _rotGrad[i];   }
   const CVector3D& GetTransGrad(int i)    const { return _transGrad[i]; }
   */

  const CVector3D& GetU()         const { return _U; }
  const CVector3D& GetV()         const { return _V; }
  bool      UseGradients()        const { return _xderivative.GetOrder()>0; }
  const CCoefVec<float>& GetCoeffs() const { return _coeffs; }
  CCoefVec<float>& GetCoeffs()    { return _coeffs; }
  const CCoefVec<float>& GetDX()  const { return _xderivative; }
  const CCoefVec<float>& GetDY()  const { return _yderivative; }
  CCoefVec<float>& GetDX()        { return _xderivative; }
  CCoefVec<float>& GetDY()        { return _yderivative; }

 protected: // data
   /*
   /// irradiance at P(N)
   CValues     _E;
   /// estimate of the standard deviation of E
   float      _stdDev;
   /// translation gradient
   CVector3D   *_transGrad;
   /// rotation gradient
   CVector3D   *_rotGrad;
   */

  /// x-axis of the local coordinate frame
  CVector3D _U;
  /// y-axis of the local coordinate frame
  CVector3D _V;
  /// (C)SH coefficients
  CCoefVec<float>    _coeffs;
  /// derivative of the coefficients along local x-axis
  CCoefVec<float>    _xderivative;
  /// derivative of the coefficients along local y-axis
  CCoefVec<float>    _yderivative;
};

__END_GOLEM_HEADER

#endif // __ICRECORD_H__
