// ===================================================================
// $Id: rt26app.h,v 1.1 2005/11/02 12:58:48 xkrivanj Exp $
//
// rt26app.cpp
//      
// Initial coding by Jaroslav Krivanek, March 2005

#ifndef __RT26APP_H__
#define __RT26APP_H__

// GOLEM headers
#include "configh.h"
#include "rt19app.h"

__BEGIN_GOLEM_HEADER

// forward declaration
class CHitInfoLite;
class CAdaptiveCaching;

/// application allocation
extern CSynthesisApp*  AllocRT_26_App();

class CRT_26_App:
  public CRT_19_App
{
public:

  /// Default constructor sets everything to zero, no construction.
  CRT_26_App();

  /// Virtual constructor calls the 'CleanUp()' method
  virtual ~CRT_26_App();

  virtual bool Run();

  virtual bool InitApplication(CWorld *world, string &outputName);
  virtual bool InitMovie(string &outputName);
  virtual bool InitFrame(string &outputName,
			 CUpdateAnimData *updateAnimdata);

  virtual bool CleanUpApplication();
  virtual bool CleanUpMovie();
  virtual bool CleanUpFrame();
 
  virtual EApplicationType GetApplicationType() const {
    return EE_AdaptiveCachingRenderer;
  }

  virtual bool NeedImagePresampling() const;

  virtual bool DoReport(const CWorld &world, ostream &app,
			CStatistics::EType reportType);
  
  /// Report only 26 app related statistics.
  void DoReportRT26(ostream &app);

  /// This is how ExtVar sampling scheme lets me know that
  /// one-sample-per-pixel pass has been finished.
  virtual bool ExtVarFSSFinishedPresampling();

protected: // methods
 
  virtual void ComputeColor(
    const CRay &ray, 
    CHitPointInfo &info,
    CValues &color, 
    bool saveForReuse = false);

  /**
     @param handleFlag (in)
       Flags as to what will be handled with adaptive caching. 
   */
  virtual void _shadeObjectDirect(
    CContributionItem &cur,
    int depth,
    int handleFlag);

  unsigned _savePixelInfo(
    CHitInfoLite &hitInfoLite,
    const CHitPointInfo &info);

  /// return the image index for current pixel
  inline int _getPixIdx() const;

protected: // data

  /// type for HDR images
  string _hdrFileType;
  /// intersection information for each image pixel
  CHitInfoLite     *_hitInfoImage;
  /// adaptive caching algo
  CAdaptiveCaching *_icAdapt;
  /// adaptive caching done for this pixel?
  bool _icAdaptDone;
  /// save HDR image of indirect illumination?
  bool _saveIndirectIllumImage;
  /// save HDR image of direct illumination?
  bool _saveDirectIllumImage;
  /// save HDR image of record positions?
  bool _saveRecPosImage;
  /// save HDR image representing the approximation error
  bool _saveApproxImage;
  /// save HDR image representing the standard deaviation
  bool _saveStdDevImage;

  /// number of tiles into which is the image subdivided for adaptive caching
  int _numTiles;

  /// saved approximation errors for irradiance cache records
  vector<float> _icA;
  /// saved approximation errors for radiance cache records
  vector<float> _rcA;
};

__END_GOLEM_HEADER

#endif // __RT26APP_H__